@@ -20,12 +20,13 @@
 
 #include <ggv-sidebar.h>
 #include <gtkchecklist.h>
-#include <gsdefaults.h>
+#include <ggv-document.h>
+
 #include <ggvutils.h>
 #include <ggv-prefs.h>
 
 struct _GgvSidebarPrivate {
-	GgvPostScriptView *ps_view;
+	GgvDocument *document;
 
 	GtkWidget *root;
 
@@ -42,6 +43,13 @@ struct _GgvSidebarClassPrivate {
 
 static BonoboControlClass *ggv_sidebar_parent_class;
 
+enum {
+	SET_PAGE_SIGNAL,
+	LAST_SIGNAL
+};
+
+static guint ggv_sidebar_signals [LAST_SIGNAL];
+
 static void
 ggv_sidebar_destroy (BonoboObject *object)
 {
@@ -86,6 +94,15 @@ ggv_sidebar_class_init (GgvSidebarClass *klass)
 	object_class->finalize = ggv_sidebar_finalize;
 
 	klass->priv = g_new0(GgvSidebarClassPrivate, 1);
+
+	ggv_sidebar_signals[SET_PAGE_SIGNAL] = g_signal_new (
+		"set_page",
+		G_TYPE_FROM_CLASS (object_class),
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (GgvSidebarClass, set_page),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__INT,
+		G_TYPE_NONE, 1, G_TYPE_INT);
 }
 
 static void
@@ -189,10 +206,12 @@ page_list_selection_changed(GtkTreeSelection *sel,
 		path = gtk_tree_model_get_path(model,
 									   &iter);
 		path_str = gtk_tree_path_to_string(path);
-		page = atoi(path_str);
+		page = atoi(path_str); /* FIXME atoi is locale dependant */
 		g_free(path_str);
 		if(page >= 0)
-			ggv_postscript_view_goto_page(sidebar->priv->ps_view, page);
+			g_signal_emit (G_OBJECT (sidebar),
+						   ggv_sidebar_signals[SET_PAGE_SIGNAL],
+						   0, page);
 		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(sidebar->priv->checklist),
 									 path, NULL, FALSE, 0.0, 0.0);
 		gtk_tree_path_free(path);
@@ -221,8 +240,8 @@ ggv_sidebar_create_page_list(GgvSidebar *sidebar)
 
 	gtk_check_list_clear(GTK_CHECK_LIST(sidebar->priv->checklist));
 
-	page_count = ggv_postscript_view_get_page_count(sidebar->priv->ps_view);
-	page_names = ggv_postscript_view_get_page_names(sidebar->priv->ps_view);
+	page_count = ggv_document_get_page_count(sidebar->priv->document);
+	page_names = ggv_document_get_page_names(sidebar->priv->document);
 	if(page_count <= 0)
 		return;
 	if(page_names == NULL)
@@ -270,17 +289,17 @@ ggv_sidebar_get_active_list(GgvSidebar *sidebar)
 BONOBO_TYPE_FUNC (GgvSidebar, BONOBO_TYPE_CONTROL, ggv_sidebar);
 
 GgvSidebar *
-ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
+ggv_sidebar_construct (GgvSidebar *sidebar, GgvDocument *doc)
 {
 	GtkWidget *hbox, *image, *sw;
 	GtkTreeSelection *sel;
 
-	g_return_val_if_fail (ps_view != NULL, NULL);
+	g_return_val_if_fail (doc != NULL, NULL);
 	g_return_val_if_fail (sidebar != NULL, NULL);
-	g_return_val_if_fail (GGV_IS_POSTSCRIPT_VIEW (ps_view), NULL);
+	g_return_val_if_fail (GGV_IS_DOCUMENT (doc), NULL);
 	g_return_val_if_fail (GGV_IS_SIDEBAR (sidebar), NULL);
 
-	sidebar->priv->ps_view = ps_view;
+	sidebar->priv->document = doc;
 
 	/* with a sidebar on the left */
 	sidebar->priv->root = gtk_vbox_new(FALSE, 0);
@@ -301,7 +320,7 @@ ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 	gtk_widget_show(hbox);
 	gtk_box_pack_start(GTK_BOX(sidebar->priv->root), hbox, FALSE, TRUE, 0);
 
-	image = gtk_image_new_from_file(GNOMEICONDIR "/ggv/toggleall.xpm");
+	image = gtk_image_new_from_stock(GGV_TOGGLE_ALL, GTK_ICON_SIZE_BUTTON);
 	gtk_widget_show(image);
 	sidebar->priv->toggle_all = gtk_button_new();
 	gtk_button_set_relief(GTK_BUTTON(sidebar->priv->toggle_all), GTK_RELIEF_NONE);
@@ -315,7 +334,7 @@ ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 	gtk_container_add(GTK_CONTAINER(sidebar->priv->toggle_all), image);
 	g_signal_connect(G_OBJECT(sidebar->priv->toggle_all), "clicked",
 					 G_CALLBACK(toggle_all_clicked), sidebar);
-	image = gtk_image_new_from_file(GNOMEICONDIR "/ggv/toggleodd.xpm");
+	image = gtk_image_new_from_stock(GGV_TOGGLE_ODD, GTK_ICON_SIZE_BUTTON);
 	gtk_widget_show(image);
 	sidebar->priv->toggle_odd = gtk_button_new();
 	gtk_tooltips_set_tip(sidebar->priv->toggle_tips, sidebar->priv->toggle_odd,
@@ -329,7 +348,7 @@ ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 	gtk_container_add(GTK_CONTAINER(sidebar->priv->toggle_odd), image);
 	g_signal_connect(G_OBJECT(sidebar->priv->toggle_odd), "clicked",
 					 G_CALLBACK(toggle_odd_clicked), sidebar);
-	image = gtk_image_new_from_file(GNOMEICONDIR "/ggv/toggleeven.xpm");
+	image = gtk_image_new_from_stock(GGV_TOGGLE_EVEN, GTK_ICON_SIZE_BUTTON);
 	gtk_widget_show(image);
 	sidebar->priv->toggle_even = gtk_button_new();
 	gtk_tooltips_set_tip(sidebar->priv->toggle_tips, sidebar->priv->toggle_even,
@@ -343,7 +362,7 @@ ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 	gtk_container_add(GTK_CONTAINER(sidebar->priv->toggle_even), image);
 	g_signal_connect(G_OBJECT(sidebar->priv->toggle_even), "clicked",
 					 G_CALLBACK(toggle_even_clicked), sidebar);
-	image = gtk_image_new_from_file(GNOMEICONDIR "/ggv/clearall.xpm");
+	image = gtk_image_new_from_stock(GGV_CLEAR_ALL, GTK_ICON_SIZE_BUTTON);
 	gtk_widget_show(image);
 	sidebar->priv->clear_all = gtk_button_new();
 	gtk_tooltips_set_tip(sidebar->priv->toggle_tips, sidebar->priv->clear_all,
@@ -381,16 +400,16 @@ ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 }
 
 GgvSidebar *
-ggv_sidebar_new (GgvPostScriptView *ps_view)
+ggv_sidebar_new (GgvDocument *doc)
 {
 	GgvSidebar *sidebar;
 	
-	g_return_val_if_fail (ps_view != NULL, NULL);
-	g_return_val_if_fail (GGV_IS_POSTSCRIPT_VIEW (ps_view), NULL);
+	g_return_val_if_fail (doc != NULL, NULL);
+	g_return_val_if_fail (GGV_IS_DOCUMENT (doc), NULL);
 
 	sidebar = g_object_new(GGV_SIDEBAR_TYPE, NULL);
 
-	return ggv_sidebar_construct (sidebar, ps_view);
+	return ggv_sidebar_construct (sidebar, doc);
 }
 
 void
