@@ -16,82 +16,239 @@
 #include "gsdefaults.h"
 #include "ggv-prefs-ui.h"
 
-static GtkWindowClass *parent_class;
+static GtkDialogClass *parent_class;
+
+static GConfClient *gconf_client = NULL;
 
 static void
-ggv_prefs_dialog_apply(GgvPrefsDialog *dlg)
+interpreter_activated(GtkWidget *entry, gpointer user_data)
 {
-        GtkWidget *active;
-        gint n, i;
-
-        gtk_widget_set_sensitive(dlg->apply, FALSE);
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
 
         gtk_gs_defaults_set_interpreter_cmd
                 (g_strdup(gtk_entry_get_text(GTK_ENTRY(dlg->gs))));
+}
+
+static void
+pdf2pscmd_activated(GtkWidget *entry, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
+
         gtk_gs_defaults_set_convert_pdf_cmd
                 (g_strdup(gtk_entry_get_text(GTK_ENTRY(dlg->convert_pdf))));
+}
+
+static void
+pdf2dsccmd_activated(GtkWidget *entry, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
+
+        gtk_gs_defaults_set_dsc_cmd
+                (g_strdup(gtk_entry_get_text(GTK_ENTRY(dlg->pdf2dsc))));
+}
+
+static void
+gzipcmd_activated(GtkWidget *entry, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
         gtk_gs_defaults_set_ungzip_cmd
                 (g_strdup(gtk_entry_get_text (GTK_ENTRY (dlg->unzip))));
+}
+
+static void
+bzip2cmd_activated(GtkWidget *entry, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
         gtk_gs_defaults_set_unbzip2_cmd
                 (g_strdup(gtk_entry_get_text (GTK_ENTRY (dlg->unbzip2))));
+}
+
+static void
+alpha_params_activated(GtkWidget *entry, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
         gtk_gs_defaults_set_alpha_parameters
                 (g_strdup(gtk_entry_get_text (GTK_ENTRY (dlg->alpha_params))));
+}
+
+static void
+scroll_step_changed(GtkWidget *entry, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
         gtk_gs_defaults_set_scroll_step
                 (gtk_spin_button_get_value(GTK_SPIN_BUTTON(dlg->scroll_step)));
+}
+
+static void
+printcmd_activated(GtkWidget *entry, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
         if(ggv_print_cmd)
                 g_free(ggv_print_cmd);
         ggv_print_cmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(dlg->print)));
+        gconf_client_set_string(gconf_client, "/apps/ggv/printing/command",
+                                ggv_print_cmd, NULL);
+}
+
+static void
+size_choice_activate(GtkWidget *choice, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
+        gint i, n;
 
-        active = gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(dlg->size))));
         n = gtk_gs_defaults_get_paper_count();
         for(i = 0; i < n; i++)
-                if(active == dlg->size_choice[i]) {
+                if(choice == dlg->size_choice[i]) {
                         gtk_gs_defaults_set_size(i);
                         break;
                 }
+}
+
+static void
+zoom_choice_activate(GtkWidget *choice, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
+        gint i;
 
-        active = gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(dlg->zoom))));
         for(i = 0; i <= ggv_max_zoom_levels; i++){
-                if(active == dlg->zoom_choice[i]) {
+                if(choice == dlg->zoom_choice[i]) {
                         gtk_gs_defaults_set_zoom_factor(ggv_zoom_levels[i]);
                         break;
                 }
         }
+}
+
+
+static void
+orientation_choice_activate(GtkWidget *choice, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
+        gint i;
 
-        active = gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(dlg->orientation))));
         for(i = 0; i <= ggv_max_orientation_labels; i++){
-                if(active == dlg->orientation_choice[i]) {
+                if(choice == dlg->orientation_choice[i]) {
                         gtk_gs_defaults_set_orientation(i);
                         break;
                 }
         }
+}
+
+
+static void
+unit_choice_activate(GtkWidget *choice, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
+        gint i;
 
-        active = gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(dlg->units))));
         for(i = 0; i <= ggv_max_unit_labels; i++){
-                if(active == dlg->unit_choice[i]) {
+                if(choice == dlg->unit_choice[i]) {
                         ggv_unit_index = i;
                         break;
                 }
         }
+        gconf_client_set_int(gconf_client, "/apps/ggv/coordinates/units",
+                             ggv_unit_index, NULL);
+}
+
+static void
+autofit_choice_activate(GtkWidget *choice, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
+        gint i;
 
-        active = gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(dlg->auto_fit))));
         for(i = 0; i <= ggv_max_auto_fit_modes; i++){
-                if(active == dlg->auto_fit_choice[i]) {
+                if(choice == dlg->auto_fit_choice[i]) {
                         gtk_gs_defaults_set_zoom_mode(i);
                         break;
                 }
         }
+}
 
+static void
+respect_eof_clicked(GtkWidget *w, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
         gtk_gs_defaults_set_respect_eof(GTK_TOGGLE_BUTTON(dlg->respect_eof)->active);
-        gtk_gs_defaults_set_watch_doc(GTK_TOGGLE_BUTTON(dlg->watch)->active);
-        gtk_gs_defaults_set_antialiased(GTK_TOGGLE_BUTTON(dlg->aa)->active);
-        gtk_gs_defaults_set_override_size(GTK_TOGGLE_BUTTON(dlg->override_size)->active);
-        gtk_gs_defaults_set_override_orientation(GTK_TOGGLE_BUTTON(dlg->override_orientation)->active);
-        gtk_gs_defaults_set_show_scroll_rect(GTK_TOGGLE_BUTTON(dlg->show_scroll_rect)->active);
+}
 
+static void
+watch_doc_clicked(GtkWidget *w, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
+        gconf_client_set_bool(gconf_client, "/apps/ggv/gtkgs/watch_doc",
+                              GTK_TOGGLE_BUTTON(dlg->watch)->active, NULL);
+}
+
+static void
+antialiasing_clicked(GtkWidget *w, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
+        gtk_gs_defaults_set_antialiased(GTK_TOGGLE_BUTTON(dlg->aa)->active);
+}
+
+static void
+override_size_clicked(GtkWidget *w, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
+        gtk_gs_defaults_set_override_size(GTK_TOGGLE_BUTTON(dlg->override_size)->active);
+}
+
+static void
+override_orientation_clicked(GtkWidget *w, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
+        gtk_gs_defaults_set_override_orientation(GTK_TOGGLE_BUTTON(dlg->override_orientation)->active);
+}
+
+static void
+show_scroll_rect_clicked(GtkWidget *w, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
+        gtk_gs_defaults_set_show_scroll_rect(GTK_TOGGLE_BUTTON(dlg->show_scroll_rect)->active);
+}
+
+
+
+static void
+show_side_panel_clicked(GtkWidget *w, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
         ggv_panel = GTK_TOGGLE_BUTTON(dlg->sidebar)->active;
+        gconf_client_set_bool(gconf_client, "/apps/ggv/layout/showpanel", 
+                              ggv_panel, NULL);
+}
+
+static void
+save_geometry_clicked(GtkWidget *w, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
         ggv_save_geometry = GTK_TOGGLE_BUTTON(dlg->savegeo)->active;
+        gconf_client_set_bool(gconf_client, "/apps/ggv/layout/savegeometry",
+                              ggv_save_geometry, NULL);
+        if(ggv_save_geometry) {
+                gconf_client_set_int(gconf_client, "/apps/ggv/layout/windowwidth",
+                                     ggv_default_width, NULL);
+                gconf_client_set_int(gconf_client, "/apps/ggv/layout/windowheight",
+                                     ggv_default_height, NULL);
+        }
+}
+
+static void
+show_toolbar_clicked(GtkWidget *w, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
 	ggv_toolbar = GTK_TOGGLE_BUTTON(dlg->toolbar)->active;
+	gconf_client_set_bool(gconf_client, "/apps/ggv/layout/showtoolbar",
+                              ggv_toolbar, NULL);
+}
+
+
+static void
+toolbar_style_toggled(GtkWidget *w, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
+        gint i;
+
         for(i = 0; i < GGV_TOOLBAR_STYLE_LAST; i++) {
                 if(GTK_TOGGLE_BUTTON(dlg->tbstyle[i])->active) {
                         ggv_toolbar_style = (GgvToolbarStyle)i;
@@ -100,17 +257,56 @@ ggv_prefs_dialog_apply(GgvPrefsDialog *dlg)
         }
         if(i == GGV_TOOLBAR_STYLE_LAST)
                 ggv_toolbar_style = GGV_TOOLBAR_STYLE_DEFAULT;
-        ggv_menubar = GTK_TOGGLE_BUTTON(dlg->mbar)->active;
-        ggv_statusbar = GTK_TOGGLE_BUTTON(dlg->sbar)->active;
-        ggv_autojump = GTK_TOGGLE_BUTTON(dlg->auto_jump)->active;
-        ggv_pageflip = GTK_TOGGLE_BUTTON(dlg->page_flip)->active;
-        ggv_right_panel = GTK_TOGGLE_BUTTON(dlg->right_panel)->active;
-
-        gtk_gs_defaults_save();
-
-        ggv_prefs_save();        
+        gconf_client_set_int(gconf_client, "/apps/ggv/layout/toolbarstyle",
+                             ggv_toolbar_style, NULL);
 }
 
+static void
+show_menubar_clicked(GtkWidget *w, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
+        ggv_menubar = GTK_TOGGLE_BUTTON(dlg->mbar)->active;
+        gconf_client_set_bool(gconf_client, "/apps/ggv/layout/showmenubar",
+                              ggv_menubar, NULL);
+}
+
+static void
+show_statusbar_clicked(GtkWidget *w, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
+        ggv_statusbar = GTK_TOGGLE_BUTTON(dlg->sbar)->active;
+	gconf_client_set_bool(gconf_client, "/apps/ggv/layout/showstatusbar",
+                              ggv_statusbar, NULL);
+}
+
+static void
+autojump_clicked(GtkWidget *w, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
+        ggv_autojump = GTK_TOGGLE_BUTTON(dlg->auto_jump)->active;
+        gconf_client_set_bool(gconf_client, "/apps/ggv/control/autojump",
+                              ggv_autojump, NULL);
+}
+
+static void
+pageflip_clicked(GtkWidget *w, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
+        ggv_pageflip = GTK_TOGGLE_BUTTON(dlg->page_flip)->active;
+        gconf_client_set_bool(gconf_client, "/apps/ggv/control/pageflip",
+                              ggv_pageflip, NULL);
+}
+
+static void
+right_panel_clicked(GtkWidget *w, gpointer user_data)
+{
+        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(user_data);
+        ggv_right_panel = GTK_TOGGLE_BUTTON(dlg->right_panel)->active;
+        gconf_client_set_bool(gconf_client, "/apps/ggv/layout/rightpanel",
+                              ggv_right_panel, NULL);
+}
+
+#if 0
 static void
 ggv_prefs_dialog_ok_clicked(GtkWidget *widget, gpointer *data)
 {
@@ -137,6 +333,7 @@ ggv_prefs_dialog_cancel_clicked(GtkWidget *widget, gpointer *data)
 
         gtk_widget_hide(GTK_WIDGET(dlg));
 }
+#endif /* 0 */
 
 static gboolean
 ggv_prefs_dialog_delete_event(GtkWidget *widget, GdkEventAny *event)
@@ -151,6 +348,9 @@ ggv_prefs_dialog_class_init(GgvPrefsDialogClass *klass)
 {
 	parent_class = g_type_class_peek_parent (klass);
 
+        if(gconf_client == NULL)
+                gconf_client = ggv_prefs_gconf_client();
+
         GTK_WIDGET_CLASS(klass)->delete_event = ggv_prefs_dialog_delete_event;
 }
 
@@ -159,20 +359,6 @@ ggv_prefs_dialog_init (GgvPrefsDialog *dlg)
 {
 }
 
-static void
-ggv_prefs_dialog_changed(GgvPrefsDialog *dlg)
-{
-        gtk_widget_set_sensitive(dlg->ok, TRUE);
-        gtk_widget_set_sensitive(dlg->apply, TRUE);
-}
-
-static void
-prefs_dlg_change_handler(GtkWidget *widget, gpointer data)
-{
-        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(data);
-        ggv_prefs_dialog_changed(dlg);
-}
-
 static void
 ggv_prefs_dialog_setup(GgvPrefsDialog *dlg)
 {
@@ -228,7 +414,7 @@ ggv_prefs_dialog_setup(GgvPrefsDialog *dlg)
 
         /* set watch document */
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dlg->watch),
-                                     gtk_gs_defaults_get_watch_doc());
+                                     ggv_watch_doc);
 
         /* set show scroll rect */
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dlg->show_scroll_rect), 
@@ -262,6 +448,9 @@ ggv_prefs_dialog_setup(GgvPrefsDialog *dlg)
         gtk_entry_set_text(GTK_ENTRY(dlg->convert_pdf),
                            gtk_gs_defaults_get_convert_pdf_cmd());
 	gtk_entry_set_position(GTK_ENTRY(dlg->convert_pdf), 0);
+        gtk_entry_set_text(GTK_ENTRY(dlg->pdf2dsc),
+                           gtk_gs_defaults_get_dsc_cmd());
+	gtk_entry_set_position(GTK_ENTRY(dlg->pdf2dsc), 0);
         gtk_entry_set_text(GTK_ENTRY(dlg->unzip),
                            gtk_gs_defaults_get_ungzip_cmd());
 	gtk_entry_set_position(GTK_ENTRY(dlg->unzip), 0);
@@ -275,11 +464,6 @@ ggv_prefs_dialog_setup(GgvPrefsDialog *dlg)
         gtk_spin_button_set_value(GTK_SPIN_BUTTON(dlg->scroll_step),
                                   gtk_gs_defaults_get_scroll_step());
 	gtk_window_set_position(GTK_WINDOW(dlg), GTK_WIN_POS_MOUSE);
-
-        gtk_widget_grab_default(dlg->cancel);
-
-        gtk_widget_set_sensitive(dlg->ok, FALSE);
-        gtk_widget_set_sensitive(dlg->apply, FALSE);
 }
 
 void
@@ -304,6 +488,24 @@ ggv_prefs_escape_pressed(GtkAccelGroup *accel_group,
         return TRUE;
 }
 
+static void
+ggv_prefs_dialog_response(GtkDialog *dlg, gint resp_id, gpointer user_data)
+{
+	GError *error = NULL;
+
+        if(GTK_RESPONSE_HELP == resp_id) {
+                gnome_help_display("ggv.xml", "ggv-settings", &error);
+                if(NULL != error) {
+                        g_warning(error->message);
+                        g_error_free(error);
+                }
+        }
+        else {
+                gtk_widget_hide(GTK_WIDGET(dlg));
+        }
+}
+
+
 GtkWidget *
 ggv_prefs_dialog_new()
 {
@@ -326,54 +528,26 @@ ggv_prefs_dialog_new()
 
         dlg = GGV_PREFS_DIALOG(g_object_new(GGV_TYPE_PREFS_DIALOG, NULL));
         widget = GTK_WIDGET(dlg);
-
-        vbox = gtk_vbox_new(FALSE, 2);
-        gtk_widget_show(vbox);
-        gtk_container_add(GTK_CONTAINER(dlg), vbox);
         gtk_container_set_border_width(GTK_CONTAINER(dlg), 2);
+        g_signal_connect(G_OBJECT(dlg), "response",
+                         G_CALLBACK(ggv_prefs_dialog_response), NULL);
 
         dlg->notebook = gtk_notebook_new();
         gtk_widget_show(dlg->notebook);
-        gtk_box_pack_start(GTK_BOX(vbox), dlg->notebook,
+        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), dlg->notebook,
                            2, TRUE, TRUE);
 
-        sep = gtk_hseparator_new();
-        gtk_widget_show(sep);
-        gtk_box_pack_start(GTK_BOX(vbox), sep, 0, FALSE, TRUE);
-
-        bbox = gtk_hbutton_box_new();
-        gtk_box_set_spacing(GTK_BOX(bbox), 4);
-        gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
-        gtk_widget_show(bbox);
-        gtk_box_pack_end(GTK_BOX(vbox), bbox, 0, FALSE, FALSE);
-        
-        dlg->cancel = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
-
         dlg_accel_group = gtk_accel_group_new();
         closure = g_cclosure_new(G_CALLBACK(ggv_prefs_escape_pressed),
                                  dlg, NULL);
         gtk_accel_group_connect(dlg_accel_group, GDK_Escape, 0, 0, closure);
         gtk_window_add_accel_group(GTK_WINDOW(dlg), dlg_accel_group);
 
-        dlg->apply = gtk_button_new_from_stock(GTK_STOCK_APPLY);
-        GTK_WIDGET_SET_FLAGS(dlg->apply, GTK_CAN_DEFAULT);
-        gtk_widget_show(dlg->apply);
-        g_signal_connect(G_OBJECT(dlg->apply), "clicked",
-                         G_CALLBACK(ggv_prefs_dialog_apply_clicked), dlg);
-        gtk_box_pack_start(GTK_BOX(bbox), dlg->apply, 0, FALSE, FALSE);
+        gtk_dialog_add_button(GTK_DIALOG(dlg), GTK_STOCK_CLOSE,
+                              GTK_RESPONSE_CLOSE);
 
-        GTK_WIDGET_SET_FLAGS(dlg->cancel, GTK_CAN_DEFAULT);
-        gtk_widget_show(dlg->cancel);
-        g_signal_connect(G_OBJECT(dlg->cancel), "clicked",
-                         G_CALLBACK(ggv_prefs_dialog_cancel_clicked), dlg);
-        gtk_box_pack_start(GTK_BOX(bbox), dlg->cancel, 0, FALSE, FALSE);
-
-        dlg->ok = gtk_button_new_from_stock(GTK_STOCK_OK);
-        GTK_WIDGET_SET_FLAGS(dlg->ok, GTK_CAN_DEFAULT);
-        gtk_widget_show(dlg->ok);
-        g_signal_connect(G_OBJECT(dlg->ok), "clicked",
-                         G_CALLBACK(ggv_prefs_dialog_ok_clicked), dlg);
-        gtk_box_pack_start(GTK_BOX(bbox), dlg->ok, 0, FALSE, FALSE);
+        gtk_dialog_add_button(GTK_DIALOG(dlg), GTK_STOCK_HELP,
+                              GTK_RESPONSE_HELP);
 
         gtk_window_set_title(GTK_WINDOW(dlg), _("GGV Preferences"));
 
@@ -409,7 +583,7 @@ ggv_prefs_dialog_new()
                 gtk_menu_shell_append(GTK_MENU_SHELL(menu), dlg->zoom_choice[i]);
                 g_signal_connect(G_OBJECT(dlg->zoom_choice[i]),
                                  "activate",
-                                 G_CALLBACK(prefs_dlg_change_handler), dlg);
+                                 G_CALLBACK(zoom_choice_activate), dlg);
         }
         gtk_option_menu_set_menu(GTK_OPTION_MENU(dlg->zoom), menu);
 
@@ -441,7 +615,7 @@ ggv_prefs_dialog_new()
                 gtk_menu_shell_append(GTK_MENU_SHELL(menu), dlg->auto_fit_choice[i]);
                 g_signal_connect(G_OBJECT(dlg->auto_fit_choice[i]),
                                  "activate",
-                                 G_CALLBACK(prefs_dlg_change_handler), dlg);
+                                 G_CALLBACK(autofit_choice_activate), dlg);
         }
         gtk_option_menu_set_menu(GTK_OPTION_MENU(dlg->auto_fit), menu);
 
@@ -453,7 +627,7 @@ ggv_prefs_dialog_new()
         gtk_label_set_mnemonic_widget(GTK_LABEL(label), dlg->auto_fit);
 
         /* units choice menu */
-       label = gtk_label_new_with_mnemonic(_("_Coordinate units:"));
+        label = gtk_label_new_with_mnemonic(_("_Coordinate units:"));
         gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
         gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
 
@@ -473,7 +647,7 @@ ggv_prefs_dialog_new()
                 gtk_menu_shell_append(GTK_MENU_SHELL(menu), dlg->unit_choice[i]);
                 g_signal_connect(G_OBJECT(dlg->unit_choice[i]),
                                  "activate",
-                                 G_CALLBACK(prefs_dlg_change_handler), dlg);
+                                 G_CALLBACK(unit_choice_activate), dlg);
         }
         gtk_option_menu_set_menu(GTK_OPTION_MENU(dlg->units), menu);
 
@@ -499,12 +673,12 @@ ggv_prefs_dialog_new()
         menu = gtk_menu_new();
         dlg->size_choice = g_new0(GtkWidget *, gtk_gs_defaults_get_paper_count());
         for(i = 0; papersizes[i].name != NULL; i++) {
-                dlg->size_choice[i] = gtk_menu_item_new_with_label(papersizes[i].name);
+                dlg->size_choice[i] = gtk_menu_item_new_with_label(_(papersizes[i].name));
                 gtk_widget_show(dlg->size_choice[i]);
                 gtk_menu_shell_append(GTK_MENU_SHELL(menu), dlg->size_choice[i]);
                 g_signal_connect(G_OBJECT(dlg->size_choice[i]),
                                  "activate",
-                                 G_CALLBACK(prefs_dlg_change_handler), dlg);
+                                 G_CALLBACK(size_choice_activate), dlg);
         }
         gtk_option_menu_set_menu(GTK_OPTION_MENU(dlg->size), menu);
 
@@ -523,7 +697,7 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
         g_signal_connect(G_OBJECT(dlg->override_size), "clicked",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
+                         G_CALLBACK(override_size_clicked), dlg);
         gtk_widget_show(dlg->override_size);
 
         /* orientation choice menu */
@@ -545,7 +719,7 @@ ggv_prefs_dialog_new()
                 gtk_menu_shell_append(GTK_MENU_SHELL(menu), dlg->orientation_choice[i]);
                 g_signal_connect(G_OBJECT(dlg->orientation_choice[i]),
                                  "activate",
-                                 G_CALLBACK(prefs_dlg_change_handler), dlg);
+                                 G_CALLBACK(orientation_choice_activate), dlg);
         }
         gtk_option_menu_set_menu(GTK_OPTION_MENU(dlg->orientation), menu);
 
@@ -564,7 +738,7 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
         g_signal_connect(G_OBJECT(dlg->override_orientation), "clicked",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
+                         G_CALLBACK(override_orientation_clicked), dlg);
         gtk_widget_show(dlg->override_orientation);
 
         /* antialiasing */
@@ -575,7 +749,7 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
         g_signal_connect(G_OBJECT(dlg->aa), "clicked",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
+                         G_CALLBACK(antialiasing_clicked), dlg);
         gtk_widget_show(dlg->aa);
 
         /* respect EOF */
@@ -586,7 +760,7 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
         g_signal_connect(G_OBJECT(dlg->respect_eof), "clicked",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
+                         G_CALLBACK(respect_eof_clicked), dlg);
         gtk_widget_show(dlg->respect_eof);
 
         /* watch file */
@@ -597,7 +771,7 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
         g_signal_connect(G_OBJECT(dlg->watch), "clicked",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
+                         G_CALLBACK(watch_doc_clicked), dlg);
         gtk_widget_show(dlg->watch);
 
         label = gtk_label_new(_("Document"));
@@ -616,7 +790,7 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
         g_signal_connect(G_OBJECT(dlg->sidebar), "clicked",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
+                         G_CALLBACK(show_side_panel_clicked), dlg);
         gtk_widget_show(dlg->sidebar);
 
         dlg->right_panel = gtk_check_button_new_with_mnemonic(_("_Put side panel on the right-hand side"));
@@ -626,7 +800,7 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
         g_signal_connect(G_OBJECT(dlg->right_panel), "clicked",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
+                         G_CALLBACK(right_panel_clicked), dlg);
         gtk_widget_show(dlg->right_panel);
 
         /* show menubar */
@@ -637,7 +811,7 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
         g_signal_connect(G_OBJECT(dlg->mbar), "clicked",
-                           G_CALLBACK(prefs_dlg_change_handler), dlg);
+                           G_CALLBACK(show_menubar_clicked), dlg);
         gtk_widget_show(dlg->mbar);
 	/* show toolbar */
         dlg->toolbar = gtk_check_button_new_with_mnemonic(_("Show _toolbar"));
@@ -647,7 +821,7 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
         g_signal_connect(G_OBJECT(dlg->toolbar), "clicked",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
+                         G_CALLBACK(show_toolbar_clicked), dlg);
         gtk_widget_show(dlg->toolbar);
 
         /* toolbar style */
@@ -666,7 +840,7 @@ ggv_prefs_dialog_new()
                                                                   _(tbstyle_labels[i]));
                 tbstyle_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(dlg->tbstyle[i]));
                 g_signal_connect(G_OBJECT(dlg->tbstyle[i]), "toggled",
-                                 G_CALLBACK(prefs_dlg_change_handler), dlg);
+                                 G_CALLBACK(toolbar_style_toggled), dlg);
                 gtk_widget_show(dlg->tbstyle[i]);
                 gtk_box_pack_start(GTK_BOX(vbox), dlg->tbstyle[i], TRUE, TRUE,
                                    GNOME_PAD_SMALL);
@@ -680,7 +854,7 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
         g_signal_connect(G_OBJECT(dlg->sbar), "clicked",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
+                         G_CALLBACK(show_statusbar_clicked), dlg);
         gtk_widget_show(dlg->sbar);
 
         /* save geometry */
@@ -691,7 +865,7 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
         g_signal_connect(G_OBJECT(dlg->savegeo), "clicked",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
+                         G_CALLBACK(save_geometry_clicked), dlg);
         gtk_widget_show(dlg->savegeo);
 
         label = gtk_label_new(_("Layout"));
@@ -711,7 +885,7 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
         g_signal_connect(G_OBJECT(dlg->auto_jump), "clicked",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
+                         G_CALLBACK(autojump_clicked), dlg);
         gtk_widget_show(dlg->auto_jump);
         /* automatically flip pages */
         dlg->page_flip = gtk_check_button_new_with_mnemonic(_("Automatically _flip pages"));
@@ -721,7 +895,7 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
         g_signal_connect(G_OBJECT(dlg->page_flip), "clicked",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
+                         G_CALLBACK(pageflip_clicked), dlg);
         gtk_widget_show(dlg->page_flip);
 
         /* show previously visible part */
@@ -732,7 +906,7 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
         g_signal_connect(G_OBJECT(dlg->show_scroll_rect), "clicked",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
+                         G_CALLBACK(show_scroll_rect_clicked), dlg);
         gtk_widget_show(dlg->show_scroll_rect);
 
         /* scroll step */
@@ -751,7 +925,7 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
         g_signal_connect(G_OBJECT(dlg->scroll_step), "value-changed",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
+                         G_CALLBACK(scroll_step_changed), dlg);
         gtk_widget_show(dlg->scroll_step);
         gtk_widget_show(label);
         gtk_label_set_mnemonic_widget(GTK_LABEL(label), dlg->scroll_step);
@@ -762,7 +936,7 @@ ggv_prefs_dialog_new()
         gtk_notebook_append_page(GTK_NOTEBOOK(dlg->notebook), table, label);
 
         /* GhostScript page */
-        table = gtk_table_new(4, 2, FALSE);
+        table = gtk_table_new(6, 2, FALSE);
 
         /* interpreter */
         label = gtk_label_new_with_mnemonic(_("_Interpreter:"));
@@ -780,9 +954,7 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 	g_signal_connect(G_OBJECT(dlg->gs), "activate",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
-	g_signal_connect(G_OBJECT(dlg->gs), "changed",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
+                         G_CALLBACK(interpreter_activated), dlg);
         gtk_widget_show(dlg->gs);
         gtk_label_set_mnemonic_widget(GTK_LABEL(label), dlg->gs);
 
@@ -802,14 +974,12 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 	g_signal_connect(G_OBJECT(dlg->alpha_params), "activate",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
-	g_signal_connect(G_OBJECT(dlg->alpha_params), "changed",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
+                         G_CALLBACK(alpha_params_activated), dlg);
         gtk_widget_show(dlg->alpha_params);
         gtk_label_set_mnemonic_widget(GTK_LABEL(label), dlg->alpha_params);
 
-        /* scan PDF command */
-        label = gtk_label_new_with_mnemonic(_("Convert _PDF to PS:"));
+        /* PDF->DSC command */
+        label = gtk_label_new_with_mnemonic(_("Convert _PDF to DSC file:"));
         gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
         gtk_table_attach(GTK_TABLE(table), label,
                          0, 1, 2, 3,
@@ -817,21 +987,19 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
         gtk_widget_show(label);
-        dlg->convert_pdf = gtk_entry_new();
-        gtk_table_attach(GTK_TABLE(table), dlg->convert_pdf,
+        dlg->pdf2dsc = gtk_entry_new();
+        gtk_table_attach(GTK_TABLE(table), dlg->pdf2dsc,
                          1, 2, 2, 3,
                          GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
-	g_signal_connect(G_OBJECT(dlg->convert_pdf), "activate",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
-	g_signal_connect(G_OBJECT(dlg->convert_pdf), "changed",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
-        gtk_widget_show(dlg->convert_pdf);
-        gtk_label_set_mnemonic_widget(GTK_LABEL(label), dlg->convert_pdf);
+	g_signal_connect(G_OBJECT(dlg->pdf2dsc), "activate",
+                         G_CALLBACK(pdf2dsccmd_activated), dlg);
+        gtk_widget_show(dlg->pdf2dsc);
+        gtk_label_set_mnemonic_widget(GTK_LABEL(label), dlg->pdf2dsc);
 
-	/* unzip command: gzip */
-        label = gtk_label_new_with_mnemonic(_("_Gzip:"));
+        /* scan PDF command */
+        label = gtk_label_new_with_mnemonic(_("Convert _PDF to PS:"));
         gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
         gtk_table_attach(GTK_TABLE(table), label,
                          0, 1, 3, 4,
@@ -839,21 +1007,19 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
         gtk_widget_show(label);
-        dlg->unzip = gtk_entry_new();
-        gtk_table_attach(GTK_TABLE(table), dlg->unzip,
+        dlg->convert_pdf = gtk_entry_new();
+        gtk_table_attach(GTK_TABLE(table), dlg->convert_pdf,
                          1, 2, 3, 4,
                          GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
-	g_signal_connect(G_OBJECT(dlg->unzip), "activate",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
-	g_signal_connect(G_OBJECT(dlg->unzip), "changed",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
-	gtk_widget_show(dlg->unzip);
-        gtk_label_set_mnemonic_widget(GTK_LABEL(label), dlg->unzip);
+	g_signal_connect(G_OBJECT(dlg->convert_pdf), "activate",
+                         G_CALLBACK(pdf2pscmd_activated), dlg);
+        gtk_widget_show(dlg->convert_pdf);
+        gtk_label_set_mnemonic_widget(GTK_LABEL(label), dlg->convert_pdf);
 
-	/* unzip command: bzip2 */
-        label = gtk_label_new_with_mnemonic(_("_Bzip2:"));
+	/* unzip command: gzip */
+        label = gtk_label_new_with_mnemonic(_("_Gzip:"));
         gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
         gtk_table_attach(GTK_TABLE(table), label,
                          0, 1, 4, 5,
@@ -861,16 +1027,34 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
         gtk_widget_show(label);
-        dlg->unbzip2 = gtk_entry_new();
-        gtk_table_attach(GTK_TABLE(table), dlg->unbzip2,
+        dlg->unzip = gtk_entry_new();
+        gtk_table_attach(GTK_TABLE(table), dlg->unzip,
                          1, 2, 4, 5,
                          GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
+	g_signal_connect(G_OBJECT(dlg->unzip), "activate",
+                         G_CALLBACK(gzipcmd_activated), dlg);
+	gtk_widget_show(dlg->unzip);
+        gtk_label_set_mnemonic_widget(GTK_LABEL(label), dlg->unzip);
+
+	/* unzip command: bzip2 */
+        label = gtk_label_new_with_mnemonic(_("_Bzip2:"));
+        gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+        gtk_table_attach(GTK_TABLE(table), label,
+                         0, 1, 5, 6,
+                         GTK_FILL,
+                         GTK_SHRINK | GTK_FILL,
+                         GNOME_PAD_SMALL, GNOME_PAD_SMALL);
+        gtk_widget_show(label);
+        dlg->unbzip2 = gtk_entry_new();
+        gtk_table_attach(GTK_TABLE(table), dlg->unbzip2,
+                         1, 2, 5, 6,
+                         GTK_EXPAND | GTK_SHRINK | GTK_FILL,
+                         GTK_SHRINK | GTK_FILL,
+                         GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 	g_signal_connect(G_OBJECT(dlg->unbzip2), "activate",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
-	g_signal_connect(G_OBJECT(dlg->unbzip2), "changed",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
+                         G_CALLBACK(bzip2cmd_activated), dlg);
 	gtk_widget_show(dlg->unbzip2);
         gtk_label_set_mnemonic_widget(GTK_LABEL(label), dlg->unbzip2);
 
@@ -897,9 +1081,11 @@ ggv_prefs_dialog_new()
                          GTK_SHRINK | GTK_FILL,
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 	g_signal_connect(G_OBJECT(dlg->print), "activate",
-                         G_CALLBACK(prefs_dlg_change_handler), dlg);
+                         G_CALLBACK(printcmd_activated), dlg);
+#if 0
 	g_signal_connect(G_OBJECT(dlg->print), "changed",
                          G_CALLBACK(prefs_dlg_change_handler), dlg);
+#endif
         gtk_widget_show(dlg->print);
         gtk_label_set_mnemonic_widget(GTK_LABEL(label), dlg->print);
 
@@ -931,7 +1117,7 @@ ggv_prefs_dialog_get_type (void)
 			(GInstanceInitFunc) ggv_prefs_dialog_init,
 		};
 		
-		ggv_prefs_dialog_type = g_type_register_static(GTK_TYPE_WINDOW, 
+		ggv_prefs_dialog_type = g_type_register_static(GTK_TYPE_DIALOG, 
                                                                "GgvPrefsDialog", 
                                                                &ggv_prefs_dialog_info, 0);
 	}
